@@ -63,7 +63,25 @@
   predict.sitar <- function(object, newdata=getData(object), level=1L, ...,
                             deriv=0L, abc=NULL,
                             xfun=function(x) x, yfun=function(y) y) {
-# create x in newdata
+# random effects
+    re <- ranef(object)
+# check if subset in call
+    subset <- eval(match.call()$subset)
+    if (!is.null(subset) && length(subset) == nrow(newdata))
+      newdata <- subset(newdata, subset)
+# check if subset from plot
+    else
+      subset <- attr(newdata, 'subset')
+# subset re
+    if (!is.null(subset)) {
+      re <- re[rownames(re) %in% getGroups(object)[subset], , drop=FALSE]
+      level <- 1L
+    }
+# centre random effects
+    re <- scale(re, scale = FALSE)
+    re.mean <- data.frame(t(attr(re, 'scaled:center')))
+    re.mean <- re.mean[rep(1, nrow(newdata)), , drop = FALSE]
+    # create x in newdata
     oc <- object$call.sitar
     x <- if ('.x' %in% names(newdata))
       newdata$.x
@@ -74,40 +92,38 @@
       warning('xoffset set to mean(x) - best to refit model')
     }
     newdata$x <- x - xoffset
-# check abc as id
-    id <- rownames(ranef(object))
-    if (all(level == 0L))
-      abc <- NULL
-    else
-    if (!is.null(abc)) {
-      if (is.null(names(abc))) {
-        if (is.vector(abc) && length(abc) == 1 && as.character(abc) %in% id) {
-          newdata$id <- abc
-          newdata$.id <- abc <- NULL
-        }
-        else
-          stop('abc unrecognised as id')
-      } else {
-# check abc as length 3 vector or data frame
-        if (is.vector(abc))
-          abc <- data.frame(t(abc))
-        if (!all(names(abc) %in% letters[1:3]))
-          stop('abc unrecognised as data frame')
-        abc[, letters[1:3][!letters[1:3] %in% names(abc)]] <- 0 # fill with zeros
-      }
-    }
+# ensure level is integral
+    level <- as.integer(level)
 # create id in newdata
+    id <- rownames(re)
     newdata$id <- if ('.id' %in% names(newdata))
       newdata$.id
     else {
-      if (any(level == 1L) && (is.null(abc) || nrow(abc) > 1))
+      if (any(level == 1L) && is.null(abc))
         eval(oc$id, newdata)
       else
         factor(1, labels=id[1])
     }
+# check if abc is id
+    if (all(level == 0L))
+      abc <- NULL
+    else if (!is.null(abc) && is.null(names(abc)) &&
+             length(abc) == 1 && as.character(abc) %in% rownames(re)) {
+      newdata$id <- abc
+      abc <- NULL
+    }
     id <- newdata$id
-    if (!is.null(abc))
-      abc <- abc[id, ] # expand with id
+# adjust abc for mean ranef
+    if (!is.null(abc)) {
+      abc.t <- re.mean
+      for (i in names(re.mean)) {
+        abc.t[, i] <- if (is.na(abc[i]))
+          re.mean[, i]
+        else
+          re.mean[, i] + abc[i]
+      }
+      abc <- abc.t
+    }
 # check if old-style object lacking fitnlme
     if (!'fitnlme' %in% names(object)) {
       warning('fitnlme missing - best to refit model')
@@ -134,17 +150,8 @@
 	        newdata[, i] <- newdata[, i] - covmeans[i]
       }
     }
-# check if newdata subsetted (from plot)
-    subset <- attr(newdata, 'subset')
+# centre covariates not in newdata to mean gd
     if (!is.null(subset)) {
-      if (!is.null(abc))
-        stop('use subset or abc but not both')
-# create abc for subset
-      re <- ranef(object)
-      abc <- apply(re[rownames(re) %in% getGroups(object)[subset], , drop=FALSE], 2, mean)
-      abc <- data.frame(t(abc))
-      level <- 1
-# set to mean gd, covariates not in newdata
       if (exists('notnames') && length(notnames) > 0) {
         if (!exists('gd'))
           gd <- update(object, returndata=TRUE)
@@ -152,25 +159,28 @@
           newdata[, i] <- mean(gd[subset, i])
       }
     }
-# set class to nlme
+# set class to nlme to use predict.nlme
     class(object) <- class(object)[-1]
-# ensure deriv integral
-    deriv <- as.integer(deriv)
-# simple prediction
-    if (all(deriv == 0L) && is.null(abc))
-      return(yfun(predict(object, newdata, level=level, ...)))
 # DISTANCE
-# level 0 prediction
-    pred0 <- yfun(predict(object, newdata, level=0L))
-    xy.id <- xyadj(object, x=x, y=0, id=id, abc=abc)
 # level 1 prediction
+    if (is.null(abc))
+      pred <- yfun(predict(object, newdata))
+    else {
+      xy.id <- xyadj(object, x=x, y=0, id=id, abc=abc)
+      newdata$x <- xy.id$x - xoffset
+      pred <- yfun(predict(object, newdata, level=0L) - xy.id$y)
+    }
+# level 0 prediction
+    xy.id <- xyadj(object, x=x, y=0, id=id, abc=re.mean)
     newdata$x <- xy.id$x - xoffset
-    pred <- yfun(predict(object, newdata, level=0L) - xy.id$y)
-    if (any(deriv > 0L)) {
+    pred0 <- yfun(predict(object, newdata, level=0L) - xy.id$y)
+# ensure deriv is unique and integral
+    deriv <- as.integer(max(deriv))
+    if (deriv > 0L) {
 # VELOCITY
 # level 0 prediction
       ss0 <- smooth.spline(xfun(x), pred0)
-      vel0 <- predict(ss0, xfun(x), deriv=max(deriv))
+      vel0 <- predict(ss0, xfun(x), deriv=deriv)
       pred0 <- vel0$y
 # velocity curve on back-transformed axes
       if (any(level == 1L)) {
@@ -179,8 +189,6 @@
             body(yfun) == as.name('x')) {
 # x and y untransformed
             vel <- spline(vel0, method='natural', xout=xfun(xy.id$x))$y
-            if (is.null(abc))
-              abc <- ranef(object)[id, , drop=FALSE]
             if (!is.null(abc$c))
               vel <- vel * exp(abc$c)
           } else {
@@ -191,9 +199,9 @@
             with(z, {
               if (length(xorig) >= 4) {
                 ss <- smooth.spline(xorig, pred, df=min(20, length(xorig)))
-                predict(ss, xorig, deriv=max(deriv))$y
+                predict(ss, xorig, deriv=deriv)$y
               } else
-                predict(ss0, xorig, deriv=max(deriv))$y
+                predict(ss0, xorig, deriv=deriv)$y
             })
           })
           vel <- do.call('c', as.list(vel))
@@ -202,7 +210,7 @@
       }
     }
 # return data frame if level 0:1
-    if (length(level) > 1)
+    if (length(level) > 1L)
       return(data.frame(id=factor(id), predict.fixed=pred0, predict.id=pred))
 # add names or split by id if level 1
     if (level == 0L)
