@@ -2,17 +2,27 @@
 #'
 #' SITAR is a method of growth curve analysis, based on \pkg{nlme}, that
 #' summarises a set of growth curves with a mean growth curve as a regression
-#' spline, plus a set of up to three fixed and random effects (a, b and c)
+#' spline, plus a set of up to four fixed and random effects (a, b, c and d)
 #' defining how individual growth curves differ from the mean curve.
 #'
+#' The SITAR model usually has up to three random effects (a, b and c), termed
+#' size, timing and intensity respectively. \code{df} sets the degrees of freedom
+#' for the mean spline curve, taking values from 1 (i.e. linear) upwards. In
+#' addition there is a random effect for the slope, d, which is fitted when
+#' \code{df = 0}, and combined with a, it provides the classic random intercept random
+#' slope model, which is similar to the 1 df spline model. In addition d can be
+#' fitted, along with a, b and c, to extend
+#' SITAR to model variability in the adult slope of the growth curve.
+#'
 #' \code{xoffset} allows the origin of \code{x} to be varied, while
 #' \code{bstart} specifies the starting value for \code{b}, both of which can
 #' affect the model fit and particularly \code{b}. The values of \code{bstart},
 #' \code{knots} and \code{bounds} are offset by \code{xoffset} for fitting
 #' purposes, and similarly for fixed effect \code{b}.
 #'
-#' The formulae \code{a.formula}, \code{b.formula} and \code{c.formula} can
-#' include functions and interactions, but \code{\link{make.names}} is used to
+#' The formulae \code{a.formula}, \code{b.formula}, \code{c.formula} and \code{d.formula}
+#' allow for cov.names and
+#' can include functions and interactions. \code{\link{make.names}} is used to
 #' ensure that the names of the corresponding model terms are valid. The
 #' modified not the original names need to be specified in \code{predict.sitar}.
 #'
@@ -27,19 +37,20 @@
 #' @param id factor of subject identifiers.
 #' @param data data frame containing variables \code{x}, \code{y} and
 #' \code{id}.
-#' @param df degrees of freedom for cubic regression spline (1 or more).
+#' @param df degrees of freedom for cubic regression spline (0 or more, see Details).
 #' @param knots vector of values for knots (default \code{df} quantiles of
 #' \code{x} distribution).
-#' @param fixed character string specifying a, b, c fixed effects (default
-#' \code{random} or the subset of "a+b+c" within \code{random}).
-#' @param random character string specifying a, b, c random effects (default
+#' @param fixed character string specifying a, b, c, d fixed effects (default
+#' \code{random} or the subset of "a + b + c + d" within \code{random}).
+#' @param random character string specifying a, b, c, d random effects (default
 #' \code{"a+b+c"}). Alternatively \code{nlme} formula e.g.
-#' \code{"list(id = pdDiag(a+b+c ~ 1))"}.
+#' \code{"list(id = pdDiag(a + b + c ~ 1))"}.
 #' @param pdDiag logical which if TRUE fits a diagonal random effects
 #' covariance matrix, or if FALSE (default) a general covariance matrix.
 #' @param a.formula formula for fixed effect a (default \code{~ 1}).
 #' @param b.formula formula for fixed effect b (default \code{~ 1}).
 #' @param c.formula formula for fixed effect c (default \code{~ 1}).
+#' @param d.formula formula for fixed effect d (default \code{~ 1}).
 #' @param bounds span of \code{x} for regression spline, or fractional
 #' extension of range (default 0.04).
 #' @param start optional numeric vector of initial estimates for the fixed
@@ -67,34 +78,32 @@
 #' @param na.action function for when the data contain NAs (see
 #' \code{\link{nlme}}).
 #' @param control list of control values for the estimation algorithm (see
-#' \code{\link{nlme}}) (default {nlmeControl(returnObject=TRUE)}).
+#' \code{\link{nlme}}) (default {nlmeControl(returnObject = TRUE)}).
 #' @param object object of class \code{sitar}.
 #' @param \dots further parameters for \code{update} consisting of any of the
 #' above \code{sitar} parameters.
 #' @param evaluate logical to control evaluation.  If TRUE (default) the
 #' expanded \code{update} call is passed to \code{sitar} for evaluation, while
 #' if FALSE the expanded call itself is returned.
+#'
 #' @return An object inheriting from class \code{sitar} representing the
 #' nonlinear mixed-effects model fit, with all the components returned by
 #' \code{nlme} (see \code{\link{nlmeObject}} for a full description) plus the
 #' following components:
 #' \item{fitnlme}{the function returning the predicted value of \code{y}.}
+#' \item{constants}{data frame of mean a-b-c-d values for unique combinations
+#' of covariates (excluding age).}
 #' \item{call.sitar}{the internal \code{sitar} call that produced the object.}
 #' \item{xoffset}{the value of \code{xoffset}.}
 #' \item{ns}{the \code{lm} object providing starting values for the B-spline curve.}
 #'
 #' Generic functions such as \code{print}, \code{plot}, \code{anova} and
 #' \code{summary} have methods to show the results of the fit. The functions
-#' \code{resid}, \code{coef}, \code{fitted}, \code{fixed.effects},
+#' \code{residuals}, \code{coef}, \code{fitted}, \code{fixed.effects},
 #' \code{random.effects}, \code{predict}, \code{getData}, \code{getGroups},
 #' \code{getCovariate} and \code{getVarCov} can be used to extract some of its
 #' components.
 #'
-#' Note that versions of \code{sitar} prior to 1.0.4 did not return
-#' \code{fitnlme}. Both \code{plot} and \code{predict} may require it, in which
-#' case they \code{update} the SITAR object on the fly, with a message. Also
-#' version 1.0.5 altered the defaults for \code{xoffset} and \code{bstart}.
-#' Models fitted with versions prior to 1.0.5 need refitting.
 #' @author Tim Cole \email{tim.cole@@ucl.ac.uk}
 #' @keywords package nonlinear regression models
 #' @examples
@@ -104,12 +113,13 @@
 #' (m1 <- sitar(x=age, y=height, id=id, data=heights, df=5))
 #'
 #' ##  relate random effects to age at menarche (with censored values +ve)
-#' ##  both a (size) and b (tempo) are positively associated with age at menarche
+#' ##  both a (size) and b (timing) are positively associated with age at menarche
 #' amen <- abs(heights$men)
-#' (m2 <- update(m1, a.form=~amen, b.form=~amen, c.form=~amen))
+#' (m2 <- update(m1, a.formula = ~amen, b.formula = ~amen, c.formula = ~amen))
 #' @import nlme
+#' @importFrom glue glue
 #' @importFrom splines ns
-#' @importFrom stats AIC BIC as.formula coef cor fitted lag lm logLik
+#' @importFrom stats AIC BIC as.formula coef cor fitted lm logLik
 #' mad model.frame model.matrix na.fail na.omit pnorm predict qnorm quantile
 #' residuals sd setNames smooth.spline spline update update.formula
 #' @export sitar
@@ -120,12 +130,13 @@ sitar <-
            data,
            df,
            knots,
-           fixed = random,
-           random = 'a+b+c',
+           fixed = NULL,
+           random = 'a + b + c',
            pdDiag = FALSE,
            a.formula =  ~ 1,
            b.formula =  ~ 1,
            c.formula =  ~ 1,
+           d.formula =  ~ 1,
            bounds = 0.04,
            start,
            xoffset = 'mean',
@@ -168,20 +179,21 @@ sitar <-
       stop("either df or knots must be specified")
     if (!missing(df) &&
         !missing(knots))
-      cat("both df and knots specified - df redefined from knots\n")
-    if (missing(knots)) {
+      warning("both df and knots specified - df redefined from knots\n")
+    if (!missing(knots)) {
+      if (!identical(range(c(knots, x)), range(x)))
+        stop("knots outside x range")
+      knots <- sort(knots)
+      df <- length(knots) + 1
+      mcall$df <- NULL
+    } else {
       df <- round(df)
-      if (df < 1)
-        stop("df must be 1 or more")
+      if (df < 0)
+        stop("df must be 0 or more")
       knots <- if (df > 1)
         quantile(x, (1:(df - 1)) / df)
       else
         numeric(0)
-    } else if (missing(df)) {
-      if (!identical(range(c(knots, x)), range(x)))
-        stop("knots outside x range")
-      knots <- knots[order(knots)]
-      df <- length(knots) + 1
     }
     if (nrow(data) <= df)
       stop("too few data to fit spline curve")
@@ -196,36 +208,75 @@ sitar <-
     knots <- knots - xoffset
     bounds <- bounds - xoffset
 
-    # get spline start values
-    spline.lm <- lm(y ~ ns(x, knots = knots, Bound = bounds))
+    # extract and format random and fixed
+    extract <- function(x, set = letters[1:4]) {
+      x <- paste(as.character(x), collapse = ' ')
+      x <- unique(strsplit(x, '[^a-z]')[[1]]) # extract words
+      paste(x[x %in% set], collapse = ' + ') # combine selected
+    }
 
-    #	if start missing get start values for ss and a
-    if (nostart <-
-        missing(start))
-      start <- coef(spline.lm)[c(2:(df + 1), 1)]
-
-    # if random contains ~ extract a+b+c for random and fixed
-    fix <- fixed
-    if (grepl('~', random)) {
-      fullrandom <- random
-      ss <- strsplit(random, '[^a-z]')[[1]] # extract words
-      ss <- paste(ss[nchar(ss) == 1], collapse='+') # combine length-1 words
-      if (fix == random)
-        fix <- ss
-      random <- ss
+    # if random contains ~ extract effects from formula
+    if (any(grepl('~', random))) {
+      fullrandom <- ifelse(is.character(random),
+                           random,
+                           deparse(random))
+      random <- extract(random)
     } else {
-      fullrandom <- if (pdDiag)
-        glue::glue('list(id = pdDiag({random} ~ 1))')
+      fullrandom <- ifelse (pdDiag,
+                            glue('list(id = pdDiag({random} ~ 1))'),
+                            NA)
+    }
+
+    # default fixed effects
+    if (is.null(fixed))
+      fixed <- random
+    # if fixed contains ~ drop it
+    else if (any(grepl('~', fixed)))
+      fixed <- as.character(as.formula(fixed))[-1]
+
+    #	force fixed effect and intercept for a
+    fixed <- extract(paste('a', fixed))
+    a.formula <- update.formula(a.formula, ~. + 1)
+
+    # adjust fixed and random for df = [01] and set start
+    if (df > 0) { # fit spline
+      ss <- paste0('s', 1:df)
+    #	if start missing get start values for ss and a
+      spline.lm <- lm(y ~ ns(x, knots = knots, Bound = bounds))
+      if (nostart <- missing(start))
+        start <- coef(spline.lm)[c(2:(df + 1), 1)]
+    # if df = 1 set fixed to a and random to [ac]
+      if (df == 1) {
+        fixed <- 'a'
+        random <- extract(random, letters[c(1, 3)])
+      }
+    # drop fixed effect for d
+      fixed <- extract(fixed, letters[1:3])
+    } else { # 0 df for spline so fit y ~ x
+      ss <- character(0)
+      #	if start missing get start values for [ad]
+      spline.lm <- lm(y ~ x)
+      if (nostart <- missing(start))
+        start <- coef(spline.lm)
+      # force fixed and random
+      fixed <- 'a + d'
+      if (!'random' %in% names(mcall))
+        random <- fixed
       else
-        NA
-  }
-    #	force fixed effect for a
-    if (!grepl('a', fix))
-      fix <- paste('a', fix, sep = '+')
+        random <- extract(random, letters[c(1, 4)])
+    }
+    if (nchar(random) == 0)
+      stop('no random effects')
 
     # set up args for fitnlme
-    fixed <- ss <- paste0('s', 1:df)
-    pars <- c('x', ss)
+    fix <- fixed
+    fixed <- ss
+    random.names <- all.names(as.formula(paste0('~', random)), functions = FALSE)
+    pars <- c('x', random.names, ss)
+
+    #	set up model elements for a, b, c and d
+    model <- setNames(letters[1:4], letters[1:4])
+    model[!names(model) %in% random.names] <- ''
 
     # if subsetted restore data
     if (!is.null(subset)) {
@@ -238,21 +289,13 @@ sitar <-
     id <- eval(mcall$id, data)
     fulldata <- data.frame(x, y, id, subset)
 
-    #	set up model elements for a, b and c
-    names(model) <- model <- letters[1:3]
-    constant <- mm.formula <- as.formula('~ 1')
+    mm.formula <- ~1
     cmm <- matrix(nrow = nrow(data), ncol = 0)
-    for (l in model) {
-      if (!grepl(l, fix) && !grepl(l, random)) {
-        model[l] <- NA
+    for (l in names(model)) {
+      formula <- update.formula(get(paste(l, 'formula', sep = '.')), ~.)
+      if ((!grepl(l, fix) && formula == ~1) || formula == ~1 - 1)
         next
-      }
-      pars <- c(pars, l)
-      formula <- get(paste(l, 'formula', sep = '.'))
-      if (formula == as.formula('~ -1') ||
-          formula == as.formula('~ 1-1') || !grepl(l, fix))
-        next
-      if (formula == constant)
+      if (formula == ~1)
         mm.intercept <- TRUE
       else {
         if (formula != mm.formula) {
@@ -283,6 +326,10 @@ sitar <-
       }
       if (mm.intercept) {
         fixed <- c(fixed, l)
+        if (!l %in% pars) {
+          pars <- c(pars, l)
+          model[l] <- paste0(l, model[l])
+        }
         if (nostart) {
           if (l == 'b')
             start <- c(start, bstart)
@@ -312,36 +359,35 @@ sitar <-
 
     if (returndata)
       return(invisible(fulldata))
-    pars <- paste(pars, collapse = ',')
+    if (nostart)
+      names(start) <- fixed
     fixed <- paste(fixed, collapse = '+')
-    sscomma <- paste(ss, collapse = ',')
+    pars <- paste(pars, collapse = ',')
+    ss <- paste(ss, collapse = ',')
 
     #	combine model elements
-    cglue <- function(x, string)
-      ifelse (is.na(x), '', string)
-    nsa <- cglue(model['a'], 'ma+')
-    nsb <- cglue(model['b'], '-mb')
-    nsc <- cglue(model['c'], ')*exp(mc')
+    cglue <- function(x, start, end)
+      ifelse (x == '', x, glue(start, x, end))
+
+    nsa <- cglue(model['a'], '', '')
+    nsb <- cglue(model['b'], '-(', ')')
+    nsc <- cglue(model['c'], ')*exp(', '')
+    nsd <- cglue(model['d'], '+(', ')*x')
+    ex <- glue('(x{nsb}{nsc})')
+    spline <- glue(cglue(ss, '+drop((cbind(', ')*ns({ex},k=knots,B=bounds))%*%mat)'))
     mat <- matrix(rep(1, df), ncol = 1)
 
     # expand fixed and if necessary random
-    fixed <- glue::glue('{fixed} ~ 1')
-    random <- if (!is.na(fullrandom))
-      fullrandom
-    else
-      glue::glue('{random} ~ 1 | id')
+    fixed <- glue('{fixed} ~ 1')
+    random <- ifelse (is.na(fullrandom),
+                      glue('{random} ~ 1 | id'),
+                      fullrandom)
 
     #	code to parse
-    fitcode <- glue::glue(
+    fitcode <- glue(
       "fitenv <- new.env()\n",
       "fitenv$fitnlme <- function(<<pars>>) {\n",
-      "ma <- <<model['a']>>\n",
-      "mb <- <<model['b']>>\n",
-      "mc <- <<model['c']>>\n",
-      "ex <- (x<<nsb>><<nsc>>)\n",
-      "ey <- <<nsa>>drop((cbind(<<sscomma>>)*ns(ex,k=knots,B=bounds))%*%mat)\n",
-      "attr(ey, 'ex') <- ex\n",
-      "ey\n",
+      "<<nsa>><<spline>><<nsd>>\n",
       "}\n",
       "on.exit(detach(fitenv))\n",
       "attach(fitenv)\n",
@@ -385,9 +431,43 @@ sitar <-
     #	save fitted model
     nlme.out <- eval(parse(text = fitcode))
     class(nlme.out) <- c('sitar', class(nlme.out))
+    # save fitnlme
     nlme.out$fitnlme <- fitenv$fitnlme
+    # save constants
+    model <- lapply(model, function(x)
+      if (x == '')
+        NULL
+      else
+        as.formula(paste0('~', x)))
+    model <- model[!sapply(model, is.null)]
+    cov.names <- unique(unlist(lapply(names(model), function(x)
+      all.names(get(paste0(x, '.formula')), functions = FALSE)
+    )))
+    random.mean <- if (is.data.frame(ranef(nlme.out)))
+      apply(ranef(nlme.out), 2, mean)
+    else # fudge for random as list
+      setNames(rep(0, length(random.names)), random.names)
+    fixed <- fixef(nlme.out)
+    fixed[random.names[!random.names %in% names(fixed)]] <- 0
+    fixed[random.names] <- fixed[random.names] + random.mean
+    newdata <- data.frame(fulldata, t(fixed))
+    model <- as.data.frame(lapply(model, function(x) eval(x[[2]], newdata)))
+    if (length(cov.names) > 0L) {
+      newdata <- setNames(as.data.frame(lapply(cov.names, function(x) with(data, get(x)))),
+                          cov.names)
+      newdata <- cbind(newdata, model)
+      for (i in rev(seq.int(cov.names)))
+        newdata <- newdata[order(newdata[, i]), ]
+    } else
+      newdata <- model
+    newdata <- unique(newdata)
+    rownames(newdata) <- 1:nrow(newdata)
+    nlme.out$constants <- newdata
+    # save call
     nlme.out$call.sitar <- mcall
+    # save xoffset
     nlme.out$xoffset <- xoffset
+    # save ns curve
     nlme.out$ns <- spline.lm
     #   if (exists('start.')) rm(start., inherits=TRUE)
     nlme.out
@@ -405,7 +485,7 @@ update.sitar <- function (object, ..., evaluate = TRUE)
   mcall$start <- NULL
   #	expand formulae
   if (any(grep('formula', names(extras)))) {
-    for (n in paste0(letters[1:3], '.formula')) {
+    for (n in paste0(letters[1:4], '.formula')) {
       if (!is.null(extras[[n]]) && !is.null(mcall[[n]]))
         extras[[n]] <- update.formula(mcall[[n]], extras[[n]])
     }
@@ -424,6 +504,7 @@ update.sitar <- function (object, ..., evaluate = TRUE)
       "a.formula",
       "b.formula",
       "c.formula",
+      "d.formula",
       "start",
       "returndata"
     ),
@@ -499,12 +580,11 @@ update.sitar <- function (object, ..., evaluate = TRUE)
       if (!is.null(extras$knots)) {
         knots <- eval(extras$knots) - xoffset
         df <- length(knots) + 1
-        mcall$df <- NULL
-      }
+         mcall$df <- NULL
+     }
       # new arg df
       else if (!is.null(extras$df)) {
         df <- eval(extras$df)
-        knots <- quantile(x, (1:(df - 1)) / df)
         mcall$knots <- NULL
       }
       # new arg bounds
@@ -515,24 +595,27 @@ update.sitar <- function (object, ..., evaluate = TRUE)
         else
           bounds <- bounds - xoffset
       }
-      #	get spline start values
-      spline.lm <-
-        lm(predict(object, data, level = 0) ~ ns(x, knots = knots, Bound = bounds))
-      start.$fixed <-
-        c(coef(spline.lm)[c(2:(df + 1), 1)], start.$fixed[fixed.extra])
-      # new arg bstart
-      if (!is.null(extras$bstart) && !is.null(start.$fixed['b'])) {
-        bstart <- eval(extras$bstart)
-        if (bstart == 'mean')
-          bstart <- mean(x)
-        else
-          bstart <- bstart - xoffset
-        start.$fixed['b'] <- bstart
+      if (df > 1 && object$ns$rank > 2) { # omit start if df was/is [01]
+        knots <- quantile(x, (1:(df - 1)) / df)
+        #	get spline start values
+        spline.lm <-
+          lm(predict(object, data, level = 0) ~ ns(x, knots = knots, Bound = bounds))
+        start.$fixed <-
+          c(coef(spline.lm)[c(2:(df + 1), 1)], start.$fixed[fixed.extra])
+        # new arg bstart
+        if (!is.null(extras$bstart) && !is.null(start.$fixed['b'])) {
+          bstart <- eval(extras$bstart)
+          if (bstart == 'mean')
+            bstart <- mean(x)
+          else
+            bstart <- bstart - xoffset
+          start.$fixed['b'] <- bstart
+        }
+        #	save start. object
+        assign('start.', start., parent.frame())
+        mcall[['start']] <- quote(start.)
       }
     }
-    #	save start. object
-    assign('start.', start., parent.frame())
-    mcall[['start']] <- quote(start.)
   }
   if (evaluate)
     eval(mcall, parent.frame())
